
import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/services.dart';

class SmallContactPicker {
  static const MethodChannel _channel =
  MethodChannel('small_contact_picker');


  /// 获取通讯录列表
  ///
  /// return list[Contact]。
  static Future<List<Contact>> get selectContacts async {
    final List result = await _channel.invokeMethod('selectContactList');
    List<Contact> contacts = [];
    for (var f in result) {
      contacts.add( Contact.fromMap(f));
    }
    return contacts;
  }

  /// 打开原生通讯录
  ///
  /// return [Contact]。
  static  Future<Contact> get selectContactWithNative async {
    final Map<dynamic, dynamic> result = await _channel.invokeMethod('selectContactNative');
    return  Contact.fromMap(result);
  }
}

/// Represents a contact selected by the user.
class Contact {
  Contact({this.fullName, this.phoneNumber, this.firstLetter,
    this.familyName,this.givenName,this.phoneArray,this.organizationName,
    this.birthday,this.thumbnailImageData,this.note
  });

  factory Contact.fromMap(Map<dynamic, dynamic> map) =>  Contact(
    fullName: map['fullName'],
    familyName: map['familyName'],
    givenName: map['givenName'],
    phoneNumber: map['phoneNumber'],
    firstLetter: map['firstLetter'],
    phoneArray: map['phoneArray'],
    organizationName: map['organizationName'],
    birthday: map['birthday'],
    thumbnailImageData: map['thumbnailImageData'],
    note: map['note'],
  );

  /// The full name of the contact, e.g. "Dr. Daniel Higgens Jr.".
  final String? fullName;
  final String? familyName;
  final String? givenName;
  final String? note;

  final int? birthday;
  // final int? birthdayYear;
  // final int? birthdayMonth;
  // final int? birthdayDay;

  /// The phone number of the contact.
  final String? phoneNumber;
  final String? organizationName;
  final Uint8List? thumbnailImageData;

  /// The firstLetter of the fullName.
  final String? firstLetter;

  final List? phoneArray;


  @override
  String toString() => '$fullName: $phoneNumber:$familyName:$givenName:$organizationName:$birthday:$phoneArray';
}